// Copyright 2025, IPng Networks GmbH, Pim van Pelt <pim@ipng.ch>

package main

import (
	"os"
	"path/filepath"
	"testing"
)

func TestNewRouterBackup(t *testing.T) {
	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "/path/to/key", 2222)

	if rb.hostname != "testhost" {
		t.Errorf("Expected hostname 'testhost', got '%s'", rb.hostname)
	}

	if rb.username != "testuser" {
		t.Errorf("Expected username 'testuser', got '%s'", rb.username)
	}

	if rb.password != "testpass" {
		t.Errorf("Expected password 'testpass', got '%s'", rb.password)
	}

	if rb.keyFile != "/path/to/key" {
		t.Errorf("Expected keyFile '/path/to/key', got '%s'", rb.keyFile)
	}

	if rb.port != 2222 {
		t.Errorf("Expected port 2222, got %d", rb.port)
	}

	if rb.client != nil {
		t.Error("Expected client to be nil initially")
	}
}

func TestRunCommandWithoutConnection(t *testing.T) {
	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	_, err := rb.RunCommand("show version")
	if err == nil {
		t.Error("Expected error when running command without connection")
	}

	if err.Error() != "no active connection" {
		t.Errorf("Expected 'no active connection' error, got '%s'", err.Error())
	}
}

func TestBackupCommandsDirectoryCreation(t *testing.T) {
	tempDir := t.TempDir()
	outputDir := filepath.Join(tempDir, "nonexistent", "backup")

	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	// This should create the directory even without a connection
	// and fail gracefully when trying to run commands
	err := rb.BackupCommands([]string{"show version"}, []string{}, outputDir)

	// Should not error on directory creation
	if _, statErr := os.Stat(outputDir); os.IsNotExist(statErr) {
		t.Error("Expected output directory to be created")
	}

	// Should return error when commands fail
	if err == nil {
		t.Error("Expected error when commands fail")
	}

	// Should NOT create the output file when commands fail (atomic behavior)
	expectedFile := filepath.Join(outputDir, "testhost")
	if _, statErr := os.Stat(expectedFile); !os.IsNotExist(statErr) {
		t.Error("Expected output file to NOT be created when commands fail")
	}
}

func TestBackupCommandsEmptyCommands(t *testing.T) {
	tempDir := t.TempDir()

	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	err := rb.BackupCommands([]string{}, []string{}, tempDir)
	if err == nil {
		t.Error("Expected error for empty commands list (no successful commands)")
	}

	// Should NOT create the output file when no commands succeed
	expectedFile := filepath.Join(tempDir, "testhost")
	if _, statErr := os.Stat(expectedFile); !os.IsNotExist(statErr) {
		t.Error("Expected output file to NOT be created when no commands succeed")
	}
}

func TestDisconnectWithoutConnection(t *testing.T) {
	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	// Should not panic or error when disconnecting without connection
	rb.Disconnect()
}

func TestFindDefaultSSHKey(t *testing.T) {
	// Test when no SSH keys exist
	originalHome := os.Getenv("HOME")
	tempDir := t.TempDir()
	os.Setenv("HOME", tempDir)
	defer os.Setenv("HOME", originalHome)

	keyPath := findDefaultSSHKey()
	if keyPath != "" {
		t.Errorf("Expected empty string when no SSH keys exist, got '%s'", keyPath)
	}

	// Create .ssh directory and a test key
	sshDir := filepath.Join(tempDir, ".ssh")
	err := os.MkdirAll(sshDir, 0700)
	if err != nil {
		t.Fatalf("Failed to create .ssh directory: %v", err)
	}

	// Create id_rsa key (should be found first)
	rsaKeyPath := filepath.Join(sshDir, "id_rsa")
	err = os.WriteFile(rsaKeyPath, []byte("fake rsa key"), 0600)
	if err != nil {
		t.Fatalf("Failed to create RSA key: %v", err)
	}

	keyPath = findDefaultSSHKey()
	if keyPath != rsaKeyPath {
		t.Errorf("Expected to find RSA key at '%s', got '%s'", rsaKeyPath, keyPath)
	}

	// Remove RSA key and create ed25519 key
	os.Remove(rsaKeyPath)
	ed25519KeyPath := filepath.Join(sshDir, "id_ed25519")
	err = os.WriteFile(ed25519KeyPath, []byte("fake ed25519 key"), 0600)
	if err != nil {
		t.Fatalf("Failed to create ed25519 key: %v", err)
	}

	keyPath = findDefaultSSHKey()
	if keyPath != ed25519KeyPath {
		t.Errorf("Expected to find ed25519 key at '%s', got '%s'", ed25519KeyPath, keyPath)
	}
}

func TestFindDefaultSSHKeyHomeError(t *testing.T) {
	// Test behavior when HOME environment is invalid
	originalHome := os.Getenv("HOME")
	os.Unsetenv("HOME")
	defer os.Setenv("HOME", originalHome)

	keyPath := findDefaultSSHKey()
	if keyPath != "" {
		t.Errorf("Expected empty string when HOME is not set, got '%s'", keyPath)
	}
}

func TestBackupCommandsFileOperations(t *testing.T) {
	tempDir := t.TempDir()

	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	// Create some fake commands (they will fail but we can test file operations)
	commands := []string{"show version", "show interfaces"}

	err := rb.BackupCommands(commands, []string{}, tempDir)
	if err == nil {
		t.Error("Expected error when all commands fail")
	}

	// Check that output file was NOT created (atomic behavior)
	outputFile := filepath.Join(tempDir, "testhost")
	_, err = os.ReadFile(outputFile)
	if err == nil {
		t.Error("Expected output file to not exist when all commands fail")
	}

	// This test verifies that atomic file behavior works correctly
}

func TestRouterBackupConnectionState(t *testing.T) {
	rb := NewRouterBackup("testhost", "", "testuser", "testpass", "", 22)

	// Initially no client
	if rb.client != nil {
		t.Error("Expected client to be nil initially")
	}

	// After disconnect, should still be nil (safe to call multiple times)
	rb.Disconnect()
	if rb.client != nil {
		t.Error("Expected client to remain nil after disconnect")
	}
}

func TestNewRouterBackupWithAddress(t *testing.T) {
	rb := NewRouterBackup("testhost", "192.168.1.100", "testuser", "testpass", "/path/to/key", 2222)

	if rb.hostname != "testhost" {
		t.Errorf("Expected hostname 'testhost', got '%s'", rb.hostname)
	}

	if rb.address != "192.168.1.100" {
		t.Errorf("Expected address '192.168.1.100', got '%s'", rb.address)
	}
}

func TestIsIPv6(t *testing.T) {
	// Test IPv4 addresses
	if isIPv6("192.168.1.1") {
		t.Error("Expected '192.168.1.1' to be detected as IPv4, not IPv6")
	}

	if isIPv6("10.0.0.1") {
		t.Error("Expected '10.0.0.1' to be detected as IPv4, not IPv6")
	}

	// Test IPv6 addresses
	if !isIPv6("2001:678:d78:500::") {
		t.Error("Expected '2001:678:d78:500::' to be detected as IPv6")
	}

	if !isIPv6("::1") {
		t.Error("Expected '::1' to be detected as IPv6")
	}

	if !isIPv6("fe80::1") {
		t.Error("Expected 'fe80::1' to be detected as IPv6")
	}

	// Test invalid addresses
	if isIPv6("invalid.address") {
		t.Error("Expected 'invalid.address' to not be detected as IPv6")
	}

	if isIPv6("hostname.example.com") {
		t.Error("Expected 'hostname.example.com' to not be detected as IPv6")
	}
}

func TestGetNetworkType(t *testing.T) {
	// Test IPv4 addresses
	if getNetworkType("192.168.1.1") != "tcp4" {
		t.Errorf("Expected 'tcp4' for IPv4 address, got '%s'", getNetworkType("192.168.1.1"))
	}

	// Test IPv6 addresses
	if getNetworkType("2001:678:d78:500::") != "tcp6" {
		t.Errorf("Expected 'tcp6' for IPv6 address, got '%s'", getNetworkType("2001:678:d78:500::"))
	}

	if getNetworkType("::1") != "tcp6" {
		t.Errorf("Expected 'tcp6' for IPv6 address, got '%s'", getNetworkType("::1"))
	}

	// Test hostnames (should default to tcp4)
	if getNetworkType("hostname.example.com") != "tcp4" {
		t.Errorf("Expected 'tcp4' for hostname, got '%s'", getNetworkType("hostname.example.com"))
	}
}

func TestIPv6AddressFormatting(t *testing.T) {
	// Test that we can create a RouterBackup with IPv6 address
	// and that it stores the address correctly
	rb := NewRouterBackup("testhost", "2001:678:d78:500::", "testuser", "testpass", "", 22)

	if !isIPv6(rb.address) {
		t.Error("Expected IPv6 address to be detected as IPv6")
	}

	if getNetworkType(rb.address) != "tcp6" {
		t.Error("Expected IPv6 address to use tcp6 network type")
	}

	// Test IPv4 for comparison
	rb4 := NewRouterBackup("testhost", "192.168.1.1", "testuser", "testpass", "", 22)

	if isIPv6(rb4.address) {
		t.Error("Expected IPv4 address to not be detected as IPv6")
	}

	if getNetworkType(rb4.address) != "tcp4" {
		t.Error("Expected IPv4 address to use tcp4 network type")
	}
}

func TestFilterOutput(t *testing.T) {
	// Test with no exclude patterns
	input := "line1\nline2\nline3"
	result := filterOutput(input, []string{})
	if result != input {
		t.Errorf("Expected no filtering with empty patterns, got '%s'", result)
	}

	// Test with matching pattern
	input = "# 2025-07-06 21:30:45 by RouterOS\nconfig line 1\nconfig line 2"
	excludePatterns := []string{"^# ....-..-.. ..:..:.. by RouterOS"}
	expected := "config line 1\nconfig line 2"
	result = filterOutput(input, excludePatterns)
	if result != expected {
		t.Errorf("Expected '%s', got '%s'", expected, result)
	}

	// Test with multiple patterns
	input = "line1\nDEBUG: debug info\nline2\nINFO: info message\nline3"
	excludePatterns = []string{"^DEBUG:", "^INFO:"}
	expected = "line1\nline2\nline3"
	result = filterOutput(input, excludePatterns)
	if result != expected {
		t.Errorf("Expected '%s', got '%s'", expected, result)
	}

	// Test with no matches
	input = "line1\nline2\nline3"
	excludePatterns = []string{"nomatch"}
	result = filterOutput(input, excludePatterns)
	if result != input {
		t.Errorf("Expected no filtering when patterns don't match, got '%s'", result)
	}
}
